import json
from django import template
from django.templatetags.static import static
from django.urls import reverse
from django.utils.safestring import mark_safe
from ..utils import random_string
from ..html_include import html_include
register = template.Library()


@register.simple_tag(takes_context=True)
def lib_include(context, *args, **kwargs):
    request = context.get('request')
    legacy = False
    if request:
        user_agent = request.META.get('HTTP_USER_AGENT', '')
        if 'Trident' in user_agent or 'MSIE' in user_agent:
            legacy = True
    include_str = ''
    if not args:
        include_str = html_include(cdn=kwargs.get('cdn'), module=kwargs.get('module'), legacy=legacy)
    for a in args:
        include_str += html_include(a, cdn=kwargs.get('cdn'), module=kwargs.get('module'), legacy=legacy)
    return mark_safe(include_str)


@register.simple_tag
def init_ajax():
    return mark_safe(f'<script src="{static("ajax_helpers/ajax_helpers.js")}"></script>')


def determine_css_class(bootstrap_style, css_class):
    if css_class:
        return css_class
    return 'btn ' + ' '.join(['btn-' + s.strip() for s in bootstrap_style.split(',')])


@register.simple_tag
def post_json_js(url_name=None, url_args=None, blob=False, **kwargs):
    json_data = {'data': kwargs}
    if url_name:
        json_data['url'] = reverse(url_name, args=url_args)
    if blob:
        json_data['response_type'] = 'blob'
    return mark_safe(f'ajax_helpers.post_json({json.dumps(json_data)})')


@register.simple_tag
def button_javascript(button_name, url_name=None, url_args=None, blob=False, **kwargs):
    return post_json_js(button=button_name, url_name=url_name, url_args=url_args, blob=blob, **kwargs)


@register.simple_tag
def ajax_button(text, name, bootstrap_style='primary', css_class=None, **kwargs):
    return mark_safe(f'''<button class="{determine_css_class(bootstrap_style, css_class) }"''' 
                     f'''onclick='{button_javascript(name, **kwargs) }'>{ text }</button>''')

@register.simple_tag
def ajax_method_button(text, method, bootstrap_style='primary', css_class=None, **kwargs):
    return mark_safe(f'''<button class="{determine_css_class(bootstrap_style, css_class) }"''' 
                     f'''onclick='{post_json_js(ajax_method=method, **kwargs) }'>{ text }</button>''')


@register.simple_tag
def send_form(form_id, **kwargs):
    kwargs.update({"form_id": form_id})
    return f'''ajax_helpers.send_form("{form_id}", {json.dumps(kwargs)})'''


@register.simple_tag
def send_form_button(text, form_id, bootstrap_style='primary', css_class=None, **kwargs):
    return mark_safe(f'''<button class="{ determine_css_class(bootstrap_style, css_class) }"''' 
                     f'''onclick='{send_form(form_id, **kwargs) }'>{ text }</button>''')


@register.inclusion_tag('ajax_helpers/upload_file.html')
def upload_file(text='Upload File', bootstrap_style='primary', css_class=None, drag_drop=None, width=None, height=None, progress=False):
    return {
        'id': random_string(),
        'text': text,
        'css_class': determine_css_class(bootstrap_style, css_class),
        'drag_drop': drag_drop,
        'width': width,
        'height': height,
        'progress': progress,
    }


@register.simple_tag
def tooltip_init(selector, function_name, placement='', template=''):
    return mark_safe(f"<script>ajax_helpers.tooltip('{selector}', '{function_name}', "
                     f"'{placement}', '{template}')</script>")


@register.inclusion_tag('ajax_helpers/ajax_timer.html')
def ajax_timer(name, interval_ms):
    return {
        'name': name,
        'interval': interval_ms
    }



import json
from asgiref.sync import async_to_sync
from ajax_helpers.utils import ajax_command
from django.core.serializers.json import DjangoJSONEncoder
try:
    from channels.generic.websocket import WebsocketConsumer
except ImportError:
    # django channels required
    pass


class ConsumerHelper(WebsocketConsumer):
    ajax_commands = ['button', 'tooltip', 'timer', 'ajax']

    def __init__(self, *args, **kwargs):
        self.slug = {}
        super().__init__(args, kwargs)
        self.group_name = None
        self.command_set = set()
        self.response_commands = []

    def add_command(self, function_name, **kwargs):
        if type(function_name) == list:
            self.response_commands += function_name
        else:
            self.response_commands.append(ajax_command(function_name, **kwargs))

    def command_response(self, function_name=None, **kwargs):
        if function_name is not None:
            self.add_command(function_name, **kwargs)
        return json.dumps({'commands': self.response_commands}, cls=DjangoJSONEncoder)

    def split_slug(self, slug):
        if slug is not None and slug != '-':
            s = slug.split('-')
            if len(s) == 1:
                self.slug['gn'] = s[0]
            else:
                self.slug.update({s[k]: s[k+1] for k in range(0, int(len(s)-1), 2)})
            if 'gn' in self.slug:
                self.group_name = self.slug['gn']

    def connect_commands(self):
        pass
        # self.add_command('message', text='Hello world')

    def pre_connect(self):
        pass  # you can modify the channel name here

    def connect(self):
        self.split_slug(self.scope['url_route']['kwargs'].get('slug'))
        # connection has to be accepted
        self.accept()
        async_to_sync(self.channel_layer.group_add)(
            self.group_name,
            self.channel_name,
        )
        self.connect_commands()
        self.send(self.command_response())

    def disconnect(self, close_code):
        async_to_sync(self.channel_layer.group_discard)(
            self.group_name,
            self.channel_name,
        )

    def receive(self, text_data=None, bytes_data=None):
        pass

    def send_commands(self, event):
        self.send(text_data=json.dumps(event))



from asgiref.sync import async_to_sync

from ajax_helpers.utils import ajax_command

try:
    from channels.layers import get_channel_layer
except ImportError:
    # django channels required
    pass


class WebsocketHelpers:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.ws_response_commands = {}
        self.channel_names = {}

    def add_ws_command(self, channel_name, function_name, **kwargs):
        if channel_name not in self.ws_response_commands:
            self.ws_response_commands[channel_name] = []
        if type(function_name) == list:
            self.ws_response_commands[channel_name] += function_name
        else:
            self.ws_response_commands[channel_name].append(ajax_command(function_name, **kwargs))

    def send_ws_commands(self, channel_name, function_name=None, **kwargs):
        if function_name is not None:
            self.add_ws_command(channel_name, function_name, **kwargs)

        commands = self.ws_response_commands.get(channel_name, [])
        channel_layer = get_channel_layer()
        async_to_sync(channel_layer.group_send)(channel_name, {"type": "send.commands", 'commands': commands})

    def add_channel(self, channel_name, ws_url=None, add_command=True):
        if ws_url is None:
            ws_url = '/helpers'
        url = f'{ws_url}/gn-{channel_name}/'
        self.channel_names[channel_name] = url
        if add_command:
            self.add_page_command('start_websocket', channel_name=channel_name, ws_url=url)



import json
from django.forms.fields import ChoiceField
from django.template.loader import render_to_string
from django.urls import reverse
from django_menus.menu import MenuItem, HtmlMenu
from django_modals.forms import CrispyForm
from django_modals.modals import FormModal


class MenuItemAjax(MenuItem):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, link_type=MenuItem.AJAX_BUTTON, **kwargs)


class MenuItemJavascript(MenuItem):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, link_type=MenuItem.JAVASCRIPT, **kwargs)


class ScreenCaptureMixin:

    def button_exit_capture(self):
        return self.command_response('redirect', url=self.request.path)

    def button_capture(self, **kwargs):
        menu = self.capture_menu('record', 'far fa-circle', 'btn-success', 'record', 'mic', 'exit')
        return self.command_response('html', selector='body', html=render_to_string(
            'ajax_helpers/screen_capture.html', {'menu': menu, 'extra_data': json.dumps(kwargs)}, self.request
        ))

    @staticmethod
    def capture_dropdown(*args):
        dropdown_items = {
            'pause': MenuItemAjax('pause', 'Pause', font_awesome='fas fa-pause'),
            'exit': MenuItemAjax('exit_capture', ' Exit Capture', font_awesome='fas fa-times'),
            'record': MenuItemAjax('record', ' Start Recording', font_awesome='fas fa-circle'),
            'upload': MenuItemAjax('upload_capture', 'Upload', font_awesome='fas fa-upload'),
            'resume': MenuItemAjax('resume', 'Resume', font_awesome='fas fa-play'),
            'mic': MenuItemJavascript(
                f"screen_capture.audio_devices('{reverse('choose_audio_modal', args=('%1%',))}')",
                ' Select Mic', font_awesome='fas fa-microphone'),
        }
        return [dropdown_items[i] for i in args]

    def capture_menu(self, function, icon, css_class, *dropdown):
        return HtmlMenu(None, 'button_group').add_items(
            MenuItemAjax(function, menu_display='', font_awesome=icon, css_classes='btn btn-sm ' + css_class,
                         dropdown=(self.capture_dropdown(*dropdown)), placement='bottom-end')).render()

    def button_recording_started(self):
        return self.command_response(
            'html', selector='.record-menu',
            html=self.capture_menu('pause', 'fas fa-pause', 'btn-danger', 'pause', 'upload', 'exit')
        )

    def button_record(self):
        return self.command_response('start_capture')

    def button_pause(self):
        self.add_command('html', selector='.record-menu',
                         html=self.capture_menu('resume', 'fas fa-play', 'btn-warning', 'resume', 'upload', 'exit'))
        return self.command_response('pause_resume')

    def button_resume(self):
        return self.command_response('pause_resume')

    def button_upload_capture(self):
        return self.command_response('upload_capture')

    def ajax_upload_video(self, **_kwargs):
        pass


class ChooseAudio(CrispyForm):
    audio_device = ChoiceField()


class ChooseAudioModal(FormModal):
    form_class = ChooseAudio
    modal_title = 'Choose Audio input'

    def form_setup(self, form, *_args, **kwargs):
        form.fields['audio_device'].choices = self.slug['base64']

    def form_valid(self, form):
        self.add_command('set_audio_source', audio_source=form.cleaned_data['audio_device'])
        return self.command_response('close')



import string
import random

from django.template.loader import render_to_string


def random_string():
    letters = string.ascii_lowercase
    return ''.join(random.choice(letters) for _i in range(8))


def ajax_command(function_name, **kwargs):
    if type(function_name) == dict:
        return function_name
    else:
        kwargs['function'] = function_name
        return kwargs


def toast_commands(*, text, header=None, header_small=None, html_id=None, position='bottom-right',
                   template_name='ajax_helpers/toast.html', header_classes='', body_classes='', auto_hide=True,
                   delay=None, font_awesome=None, show_hidden=False, extra_context=None):
    container_id = 'toast-{}'.format(position)
    container_html = '<div id="{}" style="position:fixed; {}:10px; {}:10px"></div>'.format(container_id,
                                                                                           *position.split('-'))
    if not delay and auto_hide:
        delay = 4000
    context = {'text': text,
               'header': header,
               'html_id': html_id if html_id else random_string(),
               'header_small': header_small,
               'header_classes': header_classes,
               'body_classes': body_classes,
               'auto_hide': auto_hide,
               'delay': delay,
               'font_awesome': font_awesome}
    if extra_context is not None:
        context = {**context, **extra_context}

    commands = [
        ajax_command('if_not_selector', selector='#' + container_id, commands=[
            ajax_command('append_to', selector='body', html=container_html)
        ])]
    if show_hidden:
        commands.append(ajax_command('remove', selector='#' + context['html_id'] + ':hidden'))
    commands.append(
        ajax_command('if_not_selector',
                     selector='#' + context['html_id'],
                     is_visible=show_hidden,
                     commands=[ajax_command('append_to', selector='#' + container_id,
                                            html=render_to_string(template_name=template_name, context=context))])
    )
    return commands


def is_ajax(request):
    return request.META.get('HTTP_X_REQUESTED_WITH') == 'XMLHttpRequest'
